﻿using System;
using System.Data;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;

namespace UltraGridsSynchronousScrolling
{
    public partial class Form1 : Form
    {
        private bool shiftIsPressed = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            foreach (UltraGrid grid in this.Controls)
            {
                grid.KeyDown += new KeyEventHandler(grid_KeyDown);
                grid.KeyUp += new KeyEventHandler(grid_KeyUp);
                grid.MouseDown += new MouseEventHandler(grid_MouseDown);
                grid.MouseUp += new MouseEventHandler(grid_MouseUp);
                grid.InitializeLayout += new InitializeLayoutEventHandler(grid_InitializeLayout);
                grid.BeforeRowRegionScroll += new BeforeRowRegionScrollEventHandler(grid_BeforeRowRegionScroll);
                grid.AfterRowRegionScroll += new RowScrollRegionEventHandler(grid_AfterRowRegionScroll);
                grid.AfterColRegionScroll += new ColScrollRegionEventHandler(grid_AfterColRegionScroll);
                grid.MouseWheel += new MouseEventHandler(grid_MouseWheel);

                grid.DataSource = GetData();
            }
        }

        private void grid_BeforeRowRegionScroll(object sender, BeforeRowRegionScrollEventArgs e)
        {
            // Shift is pressed, therefore the vertical scrollbar should be disabled.
            if (shiftIsPressed)
            {
                e.Cancel = true;
            }
        }

        private void grid_KeyUp(object sender, KeyEventArgs e)
        {
            shiftIsPressed = e.Shift;
        }

        private void grid_KeyDown(object sender, KeyEventArgs e)
        {
            shiftIsPressed = e.Shift;            
        }

        private void grid_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                foreach (UltraGrid grid in this.Controls)
                {
                    grid.EndUpdate();
                }
            }
        }

        private void grid_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                foreach (UltraGrid grid in this.Controls)
                {
                    grid.BeginUpdate();
                }
            }
        }

        private void grid_MouseWheel(object sender, MouseEventArgs e)
        {
            var grid = sender as UltraGrid;

            // Shift is pressed, therefore the horizontal scrollbar should be enabled.
            if (shiftIsPressed)
            {
                if (e.Delta < 0)
                {
                    ultraGrid1.ActiveColScrollRegion.Scroll(Infragistics.Win.UltraWinGrid.ColScrollAction.LineRight);
                }
                else if (e.Delta > 0)
                {
                    ultraGrid1.ActiveColScrollRegion.Scroll(Infragistics.Win.UltraWinGrid.ColScrollAction.LineLeft);
                }   
            }         
        }

        private void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.Scrollbars = Scrollbars.Both;
            e.Layout.Override.TipStyleScroll = TipStyle.Hide;
        }

        private void grid_AfterColRegionScroll(object sender, ColScrollRegionEventArgs e)
        {
            foreach (UltraGrid grid in this.Controls)
            {
                grid.ActiveColScrollRegion.Position = e.ColScrollRegion.Position;                
            }
        }

        private void grid_AfterRowRegionScroll(object sender, RowScrollRegionEventArgs e)
        {
            foreach (UltraGrid grid in this.Controls)
            {
                if (grid != (UltraGrid)sender)
                {
                    grid.ActiveRowScrollRegion.ScrollPosition = e.RowScrollRegion.ScrollPosition;
                }
            }
        }

        private DataTable GetData(int columns = 50, int rows = 25)
        {
            DataTable table = new DataTable("Table1");

            for (int index = 0; index < columns; index++)
            {
                table.Columns.Add("Column " + index, typeof(DateTime));
            }

            for (int index = 0; index < rows; index++)
            {
                DataRow row = table.NewRow();

                for (int i = 0; i < columns; i++)
                {
                    row["Column " + i] = DateTime.Now.AddHours(i);
                }

                table.Rows.Add(row);
            }

            return table;
        }
    }
}
